<?php
/**
 * @package    Joomla.Installation
 *
 * @copyright  Copyright (C) 2005 - 2020 Open Source Matters, Inc. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * HTML utility class for the installation application
 *
 * @since  1.6
 */
class InstallationHtmlHelper
{
	/**
	 * Method to generate the side bar.
	 *
	 * @return  string  Markup for the side bar.
	 *
	 * @since   1.6
	 */
	public static function stepbar()
	{
		$tabs = static::getSteps();


		$html = array();
		$html[] = '<ul class="nav nav-tabs">';

		foreach ($tabs as $tab)
		{
			$html[] = static::getTab($tab, $tabs);
		}

		$html[] = '</ul>';

		return implode('', $html);
	}

	/**
	 * Method to generate the side bar.
	 *
	 * @return  string  Markup for the side bar.
	 *
	 * @since   3.1
	 */
	public static function stepbarlanguages()
	{
		$tabs = array();
		$tabs[] = 'languages';
		$tabs[] = 'defaultlanguage';
		$tabs[] = 'complete';

		$html = array();
		$html[] = '<ul class="nav nav-tabs">';

		foreach ($tabs as $tab)
		{
			$html[] = static::getTab($tab, $tabs);
		}

		$html[] = '</ul>';

		return implode('', $html);
	}

	/**
	 * Method to generate the navigation tab.
	 *
	 * @param   string  $id    The container ID.
	 * @param   array   $tabs  The navigation tabs.
	 *
	 * @return  string  Markup for the tab.
	 *
	 * @since   3.1
	 */
	private static function getTab($id, $tabs)
	{
		$input = JFactory::getApplication()->input;
		$num   = static::getTabNumber($id, $tabs);
		$view  = static::getTabNumber($input->getWord('view'), $tabs);
		$tab   = '<span class="badge">' . $num . '</span> ' . JText::_('INSTL_STEP_' . strtoupper($id) . '_LABEL');

		if ($view + 1 === $num)
		{
			$tab = '<a href="#" onclick="Install.submitform();">' . $tab . '</a>';
		}
		elseif ($view < $num)
		{
			$tab = '<span>' . $tab . '</span>';
		}
		else
		{
			$tab = '<a href="#" onclick="return Install.goToPage(\'' . $id . '\')">' . $tab . '</a>';
		}

		return '<li class="step' . ($num === $view ? ' active' : '') . '" id="' . $id . '">' . $tab . '</li>';
	}

	/**
	 * Method to determine the tab (step) number.
	 *
	 * @param   string  $id    The container ID.
	 * @param   array   $tabs  The navigation tabs.
	 *
	 * @return  integer  Tab number in navigation sequence.
	 *
	 * @since   3.1
	 */
	private static function getTabNumber($id, $tabs)
	{
		$num = (int) array_search($id, $tabs, true);
		$num++;

		return $num;
	}

	/**
	 * Method to retrieve the available view names
	 *
	 * @return  string[]
	 *
	 * @since   1.7.2
	 */
	public static function getSteps()
	{
		// Determine if the configuration file path is writable.
		$path   = JPATH_CONFIGURATION . '/configuration.php';
		$useFtp = file_exists($path) ? !is_writable($path) : !is_writable(JPATH_CONFIGURATION . '/');

		$tabs   = array();
		$tabs[] = 'site';
		$tabs[] = 'database';

		if ($useFtp)
		{
			$tabs[] = 'ftp';
		}

		$tabs[] = 'license';
		$tabs[] = 'summary';
		$tabs[] = 'install';

		return $tabs;
	}

	/**
	 * Method to retrieve the view name relative to the given view name
	 *
	 * @param   string  $name  Current view name
	 * @param   int     $rel   Relative: -1 = previous, 0 = current, 1 = next
	 *
	 * @return  string
	 *
	 * @since   1.7.2
	 */
	public static function getStep($name, $rel = 0)
	{
		$steps = static::getSteps();
		$index = array_search($name, $steps);
		$index = $index + $rel;

		return isset($steps[$index]) ? $steps[$index] : null;
	}

	public static function getCurrencies()
	{
		$currencies = array(
			array('code_3' => 'AED', 'title' => 'AED - United Arab Emirates dirham'),
			array('code_3' => 'AFN', 'title' => 'AFN - Afghan afghani'),
			array('code_3' => 'ALL', 'title' => 'ALL - Albanian lek'),
			array('code_3' => 'AMD', 'title' => 'AMD - Armenian dram'),
			array('code_3' => 'ANG', 'title' => 'ANG - Netherlands Antillean gulden'),
			array('code_3' => 'AOA', 'title' => 'AOA - Angolan kwanza'),
			array('code_3' => 'ARS', 'title' => 'ARS - Argentine peso'),
			array('code_3' => 'AUD', 'title' => 'AUD - Australian dollar'),
			array('code_3' => 'AWG', 'title' => 'AWG - Aruban florin'),
			array('code_3' => 'AZN', 'title' => 'AZN - Azerbaijani manat'),
			array('code_3' => 'BAM', 'title' => 'BAM - Bosnia and Herzegovina convert'),
			array('code_3' => 'BBD', 'title' => 'BBD - Barbadian dollar'),
			array('code_3' => 'BDT', 'title' => 'BDT - Bangladeshi taka'),
			array('code_3' => 'BGN', 'title' => 'BGN - Bulgarian lev'),
			array('code_3' => 'BHD', 'title' => 'BHD - Bahraini dinar'),
			array('code_3' => 'BIF', 'title' => 'BIF - Burundian franc'),
			array('code_3' => 'BMD', 'title' => 'BMD - Bermudian dollar'),
			array('code_3' => 'BND', 'title' => 'BND - Brunei dollar'),
			array('code_3' => 'BOB', 'title' => 'BOB - Bolivian boliviano'),
			array('code_3' => 'BOV', 'title' => 'BOV - Mvdol'),
			array('code_3' => 'BRL', 'title' => 'BRL - Brazilian real'),
			array('code_3' => 'BSD', 'title' => 'BSD - Bahamian dollar'),
			array('code_3' => 'BTN', 'title' => 'BTN - Bhutanese ngultrum'),
			array('code_3' => 'BWP', 'title' => 'BWP - Botswana pula'),
			array('code_3' => 'BYR', 'title' => 'BYR - Belarusian ruble'),
			array('code_3' => 'BZD', 'title' => 'BZD - Belize dollar'),
			array('code_3' => 'CAD', 'title' => 'CAD - Canadian dollar'),
			array('code_3' => 'CDF', 'title' => 'CDF - Congolese franc'),
			array('code_3' => 'CHE', 'title' => 'CHE - WIR Euro'),
			array('code_3' => 'CHF', 'title' => 'CHF - Swiss franc'),
			array('code_3' => 'CHW', 'title' => 'CHW - WIR Franc'),
			array('code_3' => 'CLF', 'title' => 'CLF - Unidad de Fomento'),
			array('code_3' => 'CLP', 'title' => 'CLP - Chilean peso'),
			array('code_3' => 'CNY', 'title' => 'CNY - Chinese renminbi yuan'),
			array('code_3' => 'COP', 'title' => 'COP - Colombian peso'),
			array('code_3' => 'COU', 'title' => 'COU - Unidad de Valor Real'),
			array('code_3' => 'CRC', 'title' => 'CRC - Costa Rican colón'),
			array('code_3' => 'CUP', 'title' => 'CUP - Cuban peso'),
			array('code_3' => 'CVE', 'title' => 'CVE - Cape Verdean escudo'),
			array('code_3' => 'CZK', 'title' => 'CZK - Czech koruna'),
			array('code_3' => 'DJF', 'title' => 'DJF - Djiboutian franc'),
			array('code_3' => 'DKK', 'title' => 'DKK - Danish krone'),
			array('code_3' => 'DOP', 'title' => 'DOP - Dominican peso'),
			array('code_3' => 'DZD', 'title' => 'DZD - Algerian dinar'),
			array('code_3' => 'EGP', 'title' => 'EGP - Egyptian pound'),
			array('code_3' => 'ERN', 'title' => 'ERN - Eritrean nakfa'),
			array('code_3' => 'ETB', 'title' => 'ETB - Ethiopian birr'),
			array('code_3' => 'EUR', 'title' => 'EUR - Euro'),
			array('code_3' => 'FJD', 'title' => 'FJD - Fijian dollar'),
			array('code_3' => 'FKP', 'title' => 'FKP - Falkland pound'),
			array('code_3' => 'GBP', 'title' => 'GBP - British pound'),
			array('code_3' => 'GEL', 'title' => 'GEL - Georgian lari'),
			array('code_3' => 'GHS', 'title' => 'GHS - Ghanaian cedi'),
			array('code_3' => 'GIP', 'title' => 'GIP - Gibraltar pound'),
			array('code_3' => 'GMD', 'title' => 'GMD - Gambian dalasi'),
			array('code_3' => 'GNF', 'title' => 'GNF - Guinean franc'),
			array('code_3' => 'GTQ', 'title' => 'GTQ - Guatemalan quetzal'),
			array('code_3' => 'GYD', 'title' => 'GYD - Guyanese dollar'),
			array('code_3' => 'HKD', 'title' => 'HKD - Hong Kong dollar'),
			array('code_3' => 'HNL', 'title' => 'HNL - Honduran lempira'),
			array('code_3' => 'HRK', 'title' => 'HRK - Croatian kuna'),
			array('code_3' => 'HTG', 'title' => 'HTG - Haitian gourde'),
			array('code_3' => 'HUF', 'title' => 'HUF - Hungarian forint'),
			array('code_3' => 'IDR', 'title' => 'IDR - Indonesian rupiah'),
			array('code_3' => 'ILS', 'title' => 'ILS - Israeli new sheqel'),
			array('code_3' => 'INR', 'title' => 'INR - Indian rupee'),
			array('code_3' => 'IQD', 'title' => 'IQD - Iraqi dinar'),
			array('code_3' => 'IRR', 'title' => 'IRR - Iranian rial'),
			array('code_3' => 'ISK', 'title' => 'ISK - Icelandic króna'),
			array('code_3' => 'JMD', 'title' => 'JMD - Jamaican dollar'),
			array('code_3' => 'JOD', 'title' => 'JOD - Jordanian dinar'),
			array('code_3' => 'JPY', 'title' => 'JPY - Japanese yen'),
			array('code_3' => 'KES', 'title' => 'KES - Kenyan shilling'),
			array('code_3' => 'KGS', 'title' => 'KGS - Kyrgyzstani som'),
			array('code_3' => 'KHR', 'title' => 'KHR - Cambodian riel'),
			array('code_3' => 'KMF', 'title' => 'KMF - Comorian franc'),
			array('code_3' => 'KPW', 'title' => 'KPW - North Korean won'),
			array('code_3' => 'KRW', 'title' => 'KRW - South Korean won'),
			array('code_3' => 'KWD', 'title' => 'KWD - Kuwaiti dinar'),
			array('code_3' => 'KYD', 'title' => 'KYD - Cayman Islands dollar'),
			array('code_3' => 'KZT', 'title' => 'KZT - Kazakhstani tenge'),
			array('code_3' => 'LAK', 'title' => 'LAK - Lao kip'),
			array('code_3' => 'LBP', 'title' => 'LBP - Lebanese pound'),
			array('code_3' => 'LKR', 'title' => 'LKR - Sri Lankan rupee'),
			array('code_3' => 'LRD', 'title' => 'LRD - Liberian dollar'),
			array('code_3' => 'LSL', 'title' => 'LSL - Lesotho loti'),
			array('code_3' => 'LTL', 'title' => 'LTL - Lithuanian litas'),
			array('code_3' => 'LVL', 'title' => 'LVL - Latvian lats'),
			array('code_3' => 'LYD', 'title' => 'LYD - Libyan dinar'),
			array('code_3' => 'MAD', 'title' => 'MAD - Moroccan dirham'),
			array('code_3' => 'MDL', 'title' => 'MDL - Moldovan leu'),
			array('code_3' => 'MGA', 'title' => 'MGA - Malagasy ariary'),
			array('code_3' => 'MKD', 'title' => 'MKD - Macedonian denar'),
			array('code_3' => 'MMK', 'title' => 'MMK - Myanmar kyat'),
			array('code_3' => 'MNT', 'title' => 'MNT - Mongolian tögrög'),
			array('code_3' => 'MOP', 'title' => 'MOP - Macanese pataca'),
			array('code_3' => 'MRO', 'title' => 'MRO - Mauritanian ouguiya'),
			array('code_3' => 'MUR', 'title' => 'MUR - Mauritian rupee'),
			array('code_3' => 'MVR', 'title' => 'MVR - Maldivian rufiyaa'),
			array('code_3' => 'MWK', 'title' => 'MWK - Malawian kwacha'),
			array('code_3' => 'MXN', 'title' => 'MXN - Mexican peso'),
			array('code_3' => 'MXV', 'title' => 'MXV - Mexican Unid'),
			array('code_3' => 'MYR', 'title' => 'MYR - Malaysian ringgit'),
			array('code_3' => 'MZN', 'title' => 'MZN - Mozambican metical'),
			array('code_3' => 'NAD', 'title' => 'NAD - Namibian dollar'),
			array('code_3' => 'NGN', 'title' => 'NGN - Nigerian naira'),
			array('code_3' => 'NIO', 'title' => 'NIO - Nicaraguan córdoba'),
			array('code_3' => 'NOK', 'title' => 'NOK - Norwegian krone'),
			array('code_3' => 'NPR', 'title' => 'NPR - Nepalese rupee'),
			array('code_3' => 'NZD', 'title' => 'NZD - New Zealand dollar'),
			array('code_3' => 'OMR', 'title' => 'OMR - Omani rial'),
			array('code_3' => 'PAB', 'title' => 'PAB - Panamanian balboa'),
			array('code_3' => 'PEN', 'title' => 'PEN - Peruvian nuevo sol'),
			array('code_3' => 'PGK', 'title' => 'PGK - Papua New Guinean kina'),
			array('code_3' => 'PHP', 'title' => 'PHP - Philippine peso'),
			array('code_3' => 'PKR', 'title' => 'PKR - Pakistani rupee'),
			array('code_3' => 'PLN', 'title' => 'PLN - Polish Złoty'),
			array('code_3' => 'PYG', 'title' => 'PYG - Paraguayan guaraní'),
			array('code_3' => 'QAR', 'title' => 'QAR - Qatari riyal'),
			array('code_3' => 'RON', 'title' => 'RON - Romanian leu'),
			array('code_3' => 'RSD', 'title' => 'RSD - Serbian dinar'),
			array('code_3' => 'RUB', 'title' => 'RUB - Russian ruble'),
			array('code_3' => 'RWF', 'title' => 'RWF - Rwandan franc'),
			array('code_3' => 'SAR', 'title' => 'SAR - Saudi riyal'),
			array('code_3' => 'SBD', 'title' => 'SBD - Solomon Islands dollar'),
			array('code_3' => 'SCR', 'title' => 'SCR - Seychellois rupee'),
			array('code_3' => 'SDG', 'title' => 'SDG - Sudanese pound'),
			array('code_3' => 'SEK', 'title' => 'SEK - Swedish krona'),
			array('code_3' => 'SGD', 'title' => 'SGD - Singapore dollar'),
			array('code_3' => 'SHP', 'title' => 'SHP - Saint Helenian pound'),
			array('code_3' => 'SLL', 'title' => 'SLL - Sierra Leonean leone'),
			array('code_3' => 'SOS', 'title' => 'SOS - Somali shilling'),
			array('code_3' => 'SRD', 'title' => 'SRD - Surinamese dollar'),
			array('code_3' => 'STD', 'title' => 'STD - São Tomé and Príncipe dobra'),
			array('code_3' => 'SVC', 'title' => 'SVC - Salvadoran colón'),
			array('code_3' => 'SYP', 'title' => 'SYP - Syrian pound'),
			array('code_3' => 'SZL', 'title' => 'SZL - Swazi lilangeni'),
			array('code_3' => 'THB', 'title' => 'THB - Thai baht'),
			array('code_3' => 'TJS', 'title' => 'TJS - Tajikistani somoni'),
			array('code_3' => 'TND', 'title' => 'TND - Tunisian dinar'),
			array('code_3' => 'TOP', 'title' => 'TOP - Tongan paʻanga'),
			array('code_3' => 'TRY', 'title' => 'TRY - Turkish new lira'),
			array('code_3' => 'TTD', 'title' => 'TTD - Trinidad and Tobago dollar'),
			array('code_3' => 'TWD', 'title' => 'TWD - New Taiwan dollar'),
			array('code_3' => 'TZS', 'title' => 'TZS - Tanzanian shilling'),
			array('code_3' => 'UAH', 'title' => 'UAH - Ukrainian hryvnia'),
			array('code_3' => 'UGX', 'title' => 'UGX - Ugandan shilling'),
			array('code_3' => 'USD', 'title' => 'USD - United States dollar'),
			array('code_3' => 'UYI', 'title' => 'UYI - Uruguay Peso'),
			array('code_3' => 'UYU', 'title' => 'UYU - Uruguayan peso'),
			array('code_3' => 'UZS', 'title' => 'UZS - Uzbekistani som'),
			array('code_3' => 'VEF', 'title' => 'VEF - Venezuelan bolívar'),
			array('code_3' => 'VND', 'title' => 'VND - Vietnamese Dong'),
			array('code_3' => 'VUV', 'title' => 'VUV - Vanuatu vatu'),
			array('code_3' => 'WST', 'title' => 'WST - Samoan tala'),
			array('code_3' => 'XAF', 'title' => 'XAF - Central African CFA franc'),
			array('code_3' => 'XCD', 'title' => 'XCD - East Caribbean dollar'),
			array('code_3' => 'XOF', 'title' => 'XOF - West African CFA franc'),
			array('code_3' => 'XPF', 'title' => 'XPF - CFP franc'),
			array('code_3' => 'YER', 'title' => 'YER - Yemeni rial'),
			array('code_3' => 'ZAR', 'title' => 'ZAR - South African rand'),
			array('code_3' => 'ZMK', 'title' => 'ZMK - Zambian kwacha'),
			array('code_3' => 'ZWD', 'title' => 'ZWD - Zimbabwean dollar'),
		);

		return $currencies;
	}

	public static function getCountries()
	{
		$countries = array(
			array('id' => '73','title' => 'Afghanistan (AF)'),
			array('id' => '127','title' => 'Aland Islands (AX)'),
			array('id' => '125','title' => 'Albania (AL)'),
			array('id' => '21','title' => 'Algeria (DZ)'),
			array('id' => '220','title' => 'American Samoa (AS)'),
			array('id' => '124','title' => 'Andorra (AD)'),
			array('id' => '9','title' => 'Angola (AO)'),
			array('id' => '179','title' => 'Anguilla (AI)'),
			array('id' => '67','title' => 'Antarctica (AQ)'),
			array('id' => '178','title' => 'Antigua and Barbuda (AG)'),
			array('id' => '247','title' => 'Argentina (AR)'),
			array('id' => '74','title' => 'Armenia (AM)'),
			array('id' => '181','title' => 'Aruba (AW)'),
			array('id' => '221','title' => 'Australia (AU)'),
			array('id' => '126','title' => 'Austria (AT)'),
			array('id' => '75','title' => 'Azerbaijan (AZ)'),
			array('id' => '186','title' => 'Bahamas (BS)'),
			array('id' => '77','title' => 'Bahrain (BH)'),
			array('id' => '76','title' => 'Bangladesh (BD)'),
			array('id' => '182','title' => 'Barbados (BB)'),
			array('id' => '131','title' => 'Belarus (BY)'),
			array('id' => '129','title' => 'Belgium (BE)'),
			array('id' => '187','title' => 'Belize (BZ)'),
			array('id' => '12','title' => 'Benin (BJ)'),
			array('id' => '184','title' => 'Bermuda (BM)'),
			array('id' => '79','title' => 'Bhutan (BT)'),
			array('id' => '248','title' => 'Bolivia (BO)'),
			array('id' => '185','title' => 'Bonaire, Saint Eustatius and Saba  (BQ)'),
			array('id' => '128','title' => 'Bosnia and Herzegovina (BA)'),
			array('id' => '13','title' => 'Botswana (BW)'),
			array('id' => '68','title' => 'Bouvet Island (BV)'),
			array('id' => '249','title' => 'Brazil (BR)'),
			array('id' => '88','title' => 'British Indian Ocean Territory (IO)'),
			array('id' => '218','title' => 'British Virgin Islands (VG)'),
			array('id' => '78','title' => 'Brunei (BN)'),
			array('id' => '130','title' => 'Bulgaria (BG)'),
			array('id' => '10','title' => 'Burkina Faso (BF)'),
			array('id' => '11','title' => 'Burundi (BI)'),
			array('id' => '94','title' => 'Cambodia (KH)'),
			array('id' => '18','title' => 'Cameroon (CM)'),
			array('id' => '188','title' => 'Canada (CA)'),
			array('id' => '19','title' => 'Cape Verde (CV)'),
			array('id' => '202','title' => 'Cayman Islands (KY)'),
			array('id' => '15','title' => 'Central African Republic (CF)'),
			array('id' => '58','title' => 'Chad (TD)'),
			array('id' => '250','title' => 'Chile (CL)'),
			array('id' => '81','title' => 'China (CN)'),
			array('id' => '82','title' => 'Christmas Island (CX)'),
			array('id' => '80','title' => 'Cocos Islands (CC)'),
			array('id' => '251','title' => 'Colombia (CO)'),
			array('id' => '33','title' => 'Comoros (KM)'),
			array('id' => '222','title' => 'Cook Islands (CK)'),
			array('id' => '189','title' => 'Costa Rica (CR)'),
			array('id' => '147','title' => 'Croatia (HR)'),
			array('id' => '190','title' => 'Cuba (CU)'),
			array('id' => '191','title' => 'Curacao (CW)'),
			array('id' => '134','title' => 'Cyprus (CY)'),
			array('id' => '135','title' => 'Czech Republic (CZ)'),
			array('id' => '14','title' => 'Democratic Republic of the Congo (CD)'),
			array('id' => '137','title' => 'Denmark (DK)'),
			array('id' => '20','title' => 'Djibouti (DJ)'),
			array('id' => '192','title' => 'Dominica (DM)'),
			array('id' => '193','title' => 'Dominican Republic (DO)'),
			array('id' => '240','title' => 'East Timor (TL)'),
			array('id' => '252','title' => 'Ecuador (EC)'),
			array('id' => '22','title' => 'Egypt (EG)'),
			array('id' => '212','title' => 'El Salvador (SV)'),
			array('id' => '30','title' => 'Equatorial Guinea (GQ)'),
			array('id' => '24','title' => 'Eritrea (ER)'),
			array('id' => '138','title' => 'Estonia (EE)'),
			array('id' => '25','title' => 'Ethiopia (ET)'),
			array('id' => '253','title' => 'Falkland Islands (FK)'),
			array('id' => '141','title' => 'Faroe Islands (FO)'),
			array('id' => '223','title' => 'Fiji (FJ)'),
			array('id' => '140','title' => 'Finland (FI)'),
			array('id' => '142','title' => 'France (FR)'),
			array('id' => '254','title' => 'French Guiana (GF)'),
			array('id' => '234','title' => 'French Polynesia (PF)'),
			array('id' => '71','title' => 'French Southern Territories (TF)'),
			array('id' => '26','title' => 'Gabon (GA)'),
			array('id' => '28','title' => 'Gambia (GM)'),
			array('id' => '83','title' => 'Georgia (GE)'),
			array('id' => '136','title' => 'Germany (DE)'),
			array('id' => '27','title' => 'Ghana (GH)'),
			array('id' => '145','title' => 'Gibraltar (GI)'),
			array('id' => '146','title' => 'Greece (GR)'),
			array('id' => '195','title' => 'Greenland (GL)'),
			array('id' => '194','title' => 'Grenada (GD)'),
			array('id' => '196','title' => 'Guadeloupe (GP)'),
			array('id' => '225','title' => 'Guam (GU)'),
			array('id' => '197','title' => 'Guatemala (GT)'),
			array('id' => '144','title' => 'Guernsey (GG)'),
			array('id' => '29','title' => 'Guinea (GN)'),
			array('id' => '31','title' => 'Guinea-Bissau (GW)'),
			array('id' => '255','title' => 'Guyana (GY)'),
			array('id' => '199','title' => 'Haiti (HT)'),
			array('id' => '70','title' => 'Heard Island and McDonald Islands (HM)'),
			array('id' => '198','title' => 'Honduras (HN)'),
			array('id' => '84','title' => 'Hong Kong (HK)'),
			array('id' => '148','title' => 'Hungary (HU)'),
			array('id' => '151','title' => 'Iceland (IS)'),
			array('id' => '87','title' => 'India (IN)'),
			array('id' => '85','title' => 'Indonesia (ID)'),
			array('id' => '90','title' => 'Iran (IR)'),
			array('id' => '89','title' => 'Iraq (IQ)'),
			array('id' => '149','title' => 'Ireland (IE)'),
			array('id' => '150','title' => 'Isle of Man (IM)'),
			array('id' => '86','title' => 'Israel (IL)'),
			array('id' => '152','title' => 'Italy (IT)'),
			array('id' => '17','title' => 'Ivory Coast (CI)'),
			array('id' => '200','title' => 'Jamaica (JM)'),
			array('id' => '92','title' => 'Japan (JP)'),
			array('id' => '153','title' => 'Jersey (JE)'),
			array('id' => '91','title' => 'Jordan (JO)'),
			array('id' => '98','title' => 'Kazakhstan (KZ)'),
			array('id' => '32','title' => 'Kenya (KE)'),
			array('id' => '226','title' => 'Kiribati (KI)'),
			array('id' => '177','title' => 'Kosovo (XK)'),
			array('id' => '97','title' => 'Kuwait (KW)'),
			array('id' => '93','title' => 'Kyrgyzstan (KG)'),
			array('id' => '99','title' => 'Laos (LA)'),
			array('id' => '157','title' => 'Latvia (LV)'),
			array('id' => '100','title' => 'Lebanon (LB)'),
			array('id' => '35','title' => 'Lesotho (LS)'),
			array('id' => '34','title' => 'Liberia (LR)'),
			array('id' => '36','title' => 'Libya (LY)'),
			array('id' => '154','title' => 'Liechtenstein (LI)'),
			array('id' => '155','title' => 'Lithuania (LT)'),
			array('id' => '156','title' => 'Luxembourg (LU)'),
			array('id' => '104','title' => 'Macao (MO)'),
			array('id' => '161','title' => 'Macedonia (MK)'),
			array('id' => '38','title' => 'Madagascar (MG)'),
			array('id' => '42','title' => 'Malawi (MW)'),
			array('id' => '106','title' => 'Malaysia (MY)'),
			array('id' => '105','title' => 'Maldives (MV)'),
			array('id' => '39','title' => 'Mali (ML)'),
			array('id' => '162','title' => 'Malta (MT)'),
			array('id' => '227','title' => 'Marshall Islands (MH)'),
			array('id' => '205','title' => 'Martinique (MQ)'),
			array('id' => '40','title' => 'Mauritania (MR)'),
			array('id' => '41','title' => 'Mauritius (MU)'),
			array('id' => '63','title' => 'Mayotte (YT)'),
			array('id' => '207','title' => 'Mexico (MX)'),
			array('id' => '224','title' => 'Micronesia (FM)'),
			array('id' => '159','title' => 'Moldova (MD)'),
			array('id' => '158','title' => 'Monaco (MC)'),
			array('id' => '103','title' => 'Mongolia (MN)'),
			array('id' => '160','title' => 'Montenegro (ME)'),
			array('id' => '206','title' => 'Montserrat (MS)'),
			array('id' => '37','title' => 'Morocco (MA)'),
			array('id' => '43','title' => 'Mozambique (MZ)'),
			array('id' => '102','title' => 'Myanmar (MM)'),
			array('id' => '44','title' => 'Namibia (NA)'),
			array('id' => '231','title' => 'Nauru (NR)'),
			array('id' => '107','title' => 'Nepal (NP)'),
			array('id' => '163','title' => 'Netherlands (NL)'),
			array('id' => '180','title' => 'Netherlands Antilles (AN)'),
			array('id' => '229','title' => 'New Caledonia (NC)'),
			array('id' => '233','title' => 'New Zealand (NZ)'),
			array('id' => '208','title' => 'Nicaragua (NI)'),
			array('id' => '45','title' => 'Niger (NE)'),
			array('id' => '46','title' => 'Nigeria (NG)'),
			array('id' => '232','title' => 'Niue (NU)'),
			array('id' => '230','title' => 'Norfolk Island (NF)'),
			array('id' => '95','title' => 'North Korea (KP)'),
			array('id' => '228','title' => 'Northern Mariana Islands (MP)'),
			array('id' => '164','title' => 'Norway (NO)'),
			array('id' => '108','title' => 'Oman (OM)'),
			array('id' => '110','title' => 'Pakistan (PK)'),
			array('id' => '237','title' => 'Palau (PW)'),
			array('id' => '111','title' => 'Palestinian Territory (PS)'),
			array('id' => '209','title' => 'Panama (PA)'),
			array('id' => '235','title' => 'Papua New Guinea (PG)'),
			array('id' => '257','title' => 'Paraguay (PY)'),
			array('id' => '256','title' => 'Peru (PE)'),
			array('id' => '109','title' => 'Philippines (PH)'),
			array('id' => '236','title' => 'Pitcairn (PN)'),
			array('id' => '165','title' => 'Poland (PL)'),
			array('id' => '166','title' => 'Portugal (PT)'),
			array('id' => '211','title' => 'Puerto Rico (PR)'),
			array('id' => '112','title' => 'Qatar (QA)'),
			array('id' => '16','title' => 'Republic of the Congo (CG)'),
			array('id' => '47','title' => 'Reunion (RE)'),
			array('id' => '167','title' => 'Romania (RO)'),
			array('id' => '169','title' => 'Russia (RU)'),
			array('id' => '48','title' => 'Rwanda (RW)'),
			array('id' => '183','title' => 'Saint Barthelemy (BL)'),
			array('id' => '51','title' => 'Saint Helena (SH)'),
			array('id' => '201','title' => 'Saint Kitts and Nevis (KN)'),
			array('id' => '203','title' => 'Saint Lucia (LC)'),
			array('id' => '204','title' => 'Saint Martin (MF)'),
			array('id' => '210','title' => 'Saint Pierre and Miquelon (PM)'),
			array('id' => '217','title' => 'Saint Vincent and the Grenadines (VC)'),
			array('id' => '246','title' => 'Samoa (WS)'),
			array('id' => '174','title' => 'San Marino (SM)'),
			array('id' => '56','title' => 'Sao Tome and Principe (ST)'),
			array('id' => '113','title' => 'Saudi Arabia (SA)'),
			array('id' => '53','title' => 'Senegal (SN)'),
			array('id' => '168','title' => 'Serbia (RS)'),
			array('id' => '133','title' => 'Serbia and Montenegro (CS)'),
			array('id' => '49','title' => 'Seychelles (SC)'),
			array('id' => '52','title' => 'Sierra Leone (SL)'),
			array('id' => '114','title' => 'Singapore (SG)'),
			array('id' => '213','title' => 'Sint Maarten (SX)'),
			array('id' => '173','title' => 'Slovakia (SK)'),
			array('id' => '171','title' => 'Slovenia (SI)'),
			array('id' => '238','title' => 'Solomon Islands (SB)'),
			array('id' => '54','title' => 'Somalia (SO)'),
			array('id' => '64','title' => 'South Africa (ZA)'),
			array('id' => '69','title' => 'South Georgia and the South Sandwich Islands (GS)'),
			array('id' => '96','title' => 'South Korea (KR)'),
			array('id' => '55','title' => 'South Sudan (SS)'),
			array('id' => '139','title' => 'Spain (ES)'),
			array('id' => '101','title' => 'Sri Lanka (LK)'),
			array('id' => '50','title' => 'Sudan (SD)'),
			array('id' => '258','title' => 'Suriname (SR)'),
			array('id' => '172','title' => 'Svalbard and Jan Mayen (SJ)'),
			array('id' => '57','title' => 'Swaziland (SZ)'),
			array('id' => '170','title' => 'Sweden (SE)'),
			array('id' => '132','title' => 'Switzerland (CH)'),
			array('id' => '115','title' => 'Syria (SY)'),
			array('id' => '120','title' => 'Taiwan (TW)'),
			array('id' => '117','title' => 'Tajikistan (TJ)'),
			array('id' => '61','title' => 'Tanzania (TZ)'),
			array('id' => '116','title' => 'Thailand (TH)'),
			array('id' => '59','title' => 'Togo (TG)'),
			array('id' => '239','title' => 'Tokelau (TK)'),
			array('id' => '241','title' => 'Tonga (TO)'),
			array('id' => '215','title' => 'Trinidad and Tobago (TT)'),
			array('id' => '60','title' => 'Tunisia (TN)'),
			array('id' => '119','title' => 'Turkey (TR)'),
			array('id' => '118','title' => 'Turkmenistan (TM)'),
			array('id' => '214','title' => 'Turks and Caicos Islands (TC)'),
			array('id' => '242','title' => 'Tuvalu (TV)'),
			array('id' => '219','title' => 'U.S. Virgin Islands (VI)'),
			array('id' => '62','title' => 'Uganda (UG)'),
			array('id' => '175','title' => 'Ukraine (UA)'),
			array('id' => '72','title' => 'United Arab Emirates (AE)'),
			array('id' => '143','title' => 'United Kingdom (GB)'),
			array('id' => '216','title' => 'United States (US)'),
			array('id' => '243','title' => 'United States Minor Outlying Islands (UM)'),
			array('id' => '259','title' => 'Uruguay (UY)'),
			array('id' => '121','title' => 'Uzbekistan (UZ)'),
			array('id' => '244','title' => 'Vanuatu (VU)'),
			array('id' => '176','title' => 'Vatican (VA)'),
			array('id' => '260','title' => 'Venezuela (VE)'),
			array('id' => '122','title' => 'Vietnam (VN)'),
			array('id' => '245','title' => 'Wallis and Futuna (WF)'),
			array('id' => '23','title' => 'Western Sahara (EH)'),
			array('id' => '123','title' => 'Yemen (YE)'),
			array('id' => '65','title' => 'Zambia (ZM)'),
			array('id' => '66','title' => 'Zimbabwe (ZW)')
		);

		return $countries;
	}
}
